#! /usr/bin/python
# -*- coding: iso-8859-1 -*-
#
__author__='atareao'
__date__ ='$07/02/2011'
#
# A library for access to google weather api
#
# Copyright (C) 2011 Lorenzo Carbonell
# lorenzo.carbonell.cerezo@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import urllib2
import math
from datetime import datetime
from datetime import timedelta
from lxml import etree
from sun import Sun
from moon import Moon

GOOGLE_WEATHER_URL   = 'http://www.google.com/ig/api?weather=,,,%s,%s&hl=%s'

def f2c(temperature):
	return (float(temperature)-32.0)*5.0/9.0

def velocity_in_ms(velocity,units):
	units = units.lower().strip()
	velocity =float(velocity)
	if units == 'km/h':
		return velocity/3.6
	elif units == 'mph':
		return velocity/2.24
		

def termical_sensation(temperature,humidity,velocity):
	temperature = float(temperature)
	velocity = float(velocity)
	at = temperature - (33.0-0.04544*(33.0-temperature)*(10.45+10.0*math.pow(velocity,0.5)-velocity))
	T = temperature*9.0/5.0 + 32.0
	H = float((humidity.split(':')[1]).split('%')[0].strip())/100.0
	hi =-42.379 + 2.04901523 * T + 10.14333127 * H - 0.22475541 * T * H 
	hi = hi - 0.00683783 * math.pow(T,2.0) - 0.05481717 * math.pow(H,2.0) 
	hi = hi + 0.00122874 * math.pow(T,2.0) * H + 0.00085282 * T * math.pow(H,2) 
	hi = hi - 0.00000199 * math.pow(T,2) * math.pow(H,2)
	ts = int((f2c(hi) - at)*10.0)/10.0
	return str(ts)
	

def get_dew_point(humidity,temperature):
	# humidity (%)
	# temperature (ºC)
	humidity = (humidity.split(':')[1]).split('%')[0].strip()
	h = float(humidity)
	t = float(temperature)
	return str(int((math.pow(h/100.0,1.0/8.0)*(110.0+t) - 110.0)*10)/10.0)
	
def time_is_lower(time1,time2):
	hora1,minutos1 = time1.split(':')
	hora2,minutos2 = time2.split(':')
	hora1 = int(hora1)
	hora2 = int(hora2)
	minutos1 = int(minutos1)
	minutos2 = int(minutos2)
	if hora1 < hora2:
		return True
	elif hora1 == hora2:
		if minutos1 < minutos2:
			return True
	return False

def time_is_upper(time1,time2):
	hora1,minutos1 = time1.split(':')
	hora2,minutos2 = time2.split(':')
	hora1 = int(hora1)
	hora2 = int(hora2)
	minutos1 = int(minutos1)
	minutos2 = int(minutos2)
	if hora1 > hora2:
		return True
	elif hora1 == hora2:
		if minutos1 > minutos2:
			return True
	return False
	
def get_text(root,path):	
	r = root.xpath(path)
	if len(r)>0:
		return r[0].text
	return None

def get_data(root,path):	
	r = root.xpath(path)
	if len(r)>0:
		return r[0].attrib['data']
	return None

def get_moon_icon(day):
	moon = Moon(day)
	return moon.icon()
	
def get_moon_phase(day):
	moon = Moon(day)
	return moon.phase()
	
	
def is_day_now(sunrise,sunset):
	hora = ('%s:%s')%(datetime.time(datetime.now()).hour,datetime.time(datetime.now()).minute)
	if time_is_lower(hora,sunrise) or time_is_upper(hora,sunset):
		return False
	return True
	
def get_sunrise(day,longitude,latitude):
	sun = Sun()
	ss = sun.sunRiseSetLocal(day.year, day.month,day.day, float(longitude),float(latitude))
	return '%s'%(ss[0])
		
def get_sunset(day,longitude,latitude):
	sun = Sun()
	ss = sun.sunRiseSetLocal(day.year, day.month,day.day, float(longitude),float(latitude))
	return '%s'%(ss[1])

def get_weather_from_google(latitude = 39.4702393, longitude = -0.3768049, hl = ''):
	lat = int(latitude * 1000000)
	lon = int(longitude * 1000000)
	url = GOOGLE_WEATHER_URL % (lat,lon,hl)
	#
	try:
		handler = urllib2.urlopen(urllib2.Request(url, headers={'User-Agent' : "Magic Browser"}))
		charset = (handler.info().plist[0].split('=')[1])
		if not charset:
			charset = 'utf-8'
		if charset.lower() != 'utf-8':
			xml_response = handler.read().decode(charset).encode('utf-8')
		else:
			xml_response = handler.read()
		handler.close()
		#
		weather_data = {}
		#
		root = etree.fromstring(xml_response).xpath('/xml_api_reply/weather/current_conditions')
		current_conditions = {}
		current_conditions['condition'] = get_data(root[0],'condition')
		current_conditions['temp_f'] = get_data(root[0],'temp_f')
		current_conditions['temp_c'] = get_data(root[0],'temp_c')
		current_conditions['humidity'] = get_data(root[0],'humidity')
		current_conditions['dew_point'] = get_dew_point(current_conditions['humidity'],current_conditions['temp_c'])
		current_conditions['icon'] = get_data(root[0],'icon')
		current_conditions['wind_condition'] = get_data(root[0],'wind_condition')
		velocity = velocity_in_ms(current_conditions['wind_condition'].split(' ')[-2],current_conditions['wind_condition'].split(' ')[-1])
		current_conditions['termical_sensation'] = termical_sensation(current_conditions['temp_c'],current_conditions['humidity'],velocity)
		current_conditions['sunrise'] = get_sunrise(datetime.today(),longitude,latitude)
		current_conditions['sunset'] = get_sunset(datetime.today(),longitude,latitude)
		current_conditions['moon_icon'] = get_moon_icon(datetime.today())
		current_conditions['moon_phase'] = get_moon_phase(datetime.today())
		#
		root = etree.fromstring(xml_response).xpath('/xml_api_reply/weather/forecast_conditions')
		forecast_conditions = []
		dia = datetime.today()
		undia = timedelta(days=1)
		for el in root:
			fc = {}
			fc['day_of_week'] = dia.strftime('%A').capitalize()
			fc['low'] = get_data(el,'low')
			fc['high'] = get_data(el,'high')
			fc['icon'] = get_data(el,'icon')
			fc['condition'] = get_data(el,'condition')
			fc['sunrise'] = get_sunrise(dia,longitude,latitude)
			fc['sunset'] = get_sunset(dia,longitude,latitude)	
			fc['moon_icon'] = get_moon_icon(dia)
			fc['moon_phase'] = get_moon_phase(dia)
			forecast_conditions.append(fc)
			dia = dia + undia
		#
		root = etree.fromstring(xml_response).xpath('/xml_api_reply/weather/forecast_information')
		forecast_information = {}
		forecast_information['city'] = get_data(root[0],'city')
		forecast_information['postal_code'] = get_data(root[0],'postal_code')
		forecast_information['latitude_e6'] = get_data(root[0],'latitude_e6')
		forecast_information['longitude_e6'] = get_data(root[0],'longitude_e6')
		forecast_information['forecast_date'] = get_data(root[0],'forecast_date')
		forecast_information['current_date_time'] = get_data(root[0],'current_date_time')
		forecast_information['unit_system'] = get_data(root[0],'unit_system')
		#
		url = GOOGLE_WEATHER_URL % (lat,lon,'en')
		#
		handler = urllib2.urlopen(urllib2.Request(url, headers={'User-Agent' : "Magic Browser"}))
		charset = (handler.info().plist[0].split('=')[1])
		if not charset:
			charset = 'utf-8'
		if charset.lower() != 'utf-8':
			xml_response = handler.read().decode(charset).encode('utf-8')
		else:
			xml_response = handler.read()
		handler.close()
		#
		root = etree.fromstring(xml_response).xpath('/xml_api_reply/weather/current_conditions')
		status = get_data(root[0],'condition')
		current_conditions['condition_en'] = status
		is_day = is_day_now(current_conditions['sunrise'],current_conditions['sunset'])
		current_conditions['status_icon'] = from_status_to_icon_name(status,is_day)
		current_conditions['status_image'] = from_status_to_image(status,is_day)
		#
		root = etree.fromstring(xml_response).xpath('/xml_api_reply/weather/forecast_conditions')
		i=0
		for el in root:
			status = get_data(el,'condition')
			forecast_conditions[i]['condition_en'] = status
			forecast_conditions[i]['status_icon'] = from_status_to_icon_name(status,True)
			forecast_conditions[i]['status_image'] = from_status_to_image(status,True)
			i += 1
	except urllib2.URLError,e:
		weather_data = {}
		#
		current_conditions = {}
		current_conditions['condition'] = ''
		current_conditions['temp_f'] = ''
		current_conditions['temp_c'] = ''
		current_conditions['humidity'] = ''
		current_conditions['dew_point'] = ''
		current_conditions['icon'] = 'na.png'
		current_conditions['wind_condition'] = ''
		current_conditions['termical_sensation'] = ''
		current_conditions['sunrise'] = get_sunrise(datetime.today(),longitude,latitude)
		current_conditions['sunset'] = get_sunset(datetime.today(),longitude,latitude)
		current_conditions['moon_icon'] = get_moon_icon(datetime.today())
		current_conditions['moon_phase'] = get_moon_phase(datetime.today())
		current_conditions['condition_en'] = ''
		current_conditions['status_icon'] = 'weather-severe-alert'
		current_conditions['status_image'] = 'na.png'		
		#
		forecast_conditions = []
		dia = datetime.today()
		undia = timedelta(days=1)
		for i in range(0,4):
			fc = {}
			fc['day_of_week'] = dia.strftime('%A').capitalize()
			fc['low'] = ''
			fc['high'] = ''
			fc['icon'] = 'na.png'
			fc['condition'] = ''
			fc['sunrise'] = get_sunrise(dia,longitude,latitude)
			fc['sunset'] = get_sunset(dia,longitude,latitude)	
			fc['moon_icon'] = get_moon_icon(dia)
			fc['moon_phase'] = get_moon_phase(dia)
			fc['condition_en'] = ''
			fc['status_icon'] = 'weather-severe-alert'
			fc['status_image'] = 'na.png'
			forecast_conditions.append(fc)
			dia = dia + undia
		#
		forecast_information = {}
		forecast_information['city'] = '...'
		forecast_information['postal_code'] = ''
		forecast_information['latitude_e6'] = ''
		forecast_information['longitude_e6'] = ''
		forecast_information['forecast_date'] = ''
		forecast_information['current_date_time'] = ''
		forecast_information['unit_system'] = 'SI'
	#
	weather_data['current_conditions'] = current_conditions
	weather_data['forecasts'] = forecast_conditions
	weather_data['forecast_information'] = forecast_information
	#
	return weather_data
	
def from_status_to_image(status,day = True):
	EXT = '.png'
	if day == True:
		DON_EXT = EXT
	else:
		DON_EXT = '_n' + EXT
	status = status.lower()
	if status.find('rain') !=-1 and status.find('chance') != -1:		
		return 'chance_of_rain' + DON_EXT
	elif status.find('snow') !=-1 and status.find('chance') != -1:		
		return 'chance_of_snow' + DON_EXT
	elif status.find('storm') !=-1 and status.find('chance') != -1:		
		return 'chance_of_storm' + DON_EXT
	elif status.find('tstorm') !=-1 and status.find('chance') != -1:		
		return 'chance_of_tstorm' + DON_EXT
	elif status.find('cloudy') !=-1 and status.find('mostly') != -1:		
		return 'mostly_cloudy' + DON_EXT
	elif status.find('cloudy') !=-1 and status.find('partly') != -1:		
		return 'partly_cloudy' + DON_EXT
	elif status.find('sunny') !=-1 and status.find('mostly') != -1:		
		return 'mostly_sunny' + DON_EXT
	elif status.find('cloudy') != -1:
		return 'cloudy' + EXT
	elif status.find('dust') != -1:
		return 'dust' + EXT
	elif status.find('flurries') != -1:
		return 'flurries' + EXT
	elif status.find('fog') != -1:
		return 'fog' + EXT
	elif status.find('haze') != -1:
		return 'haze' + EXT
	elif status.find('icy') != -1:
		return 'icy' + EXT
	elif status.find('mist') != -1:
		return 'mist' + EXT
	elif status.find('mist') != -1:
		return 'mist' + EXT
	elif status.find('rain') != -1:
		return 'rain' + EXT
	elif status.find('sleet') != -1:
		return 'sleet' + EXT
	elif status.find('smoke') != -1:
		return 'smoke' + EXT
	elif status.find('snow') != -1:
		return 'snow' + EXT
	elif status.find('storm') != -1:
		return 'storm' + EXT
	elif status.find('thunderstorm') != -1:
		return 'thunderstorm' + EXT
	elif status.find('sunny') != -1:
		return 'sunny' + DON_EXT
	return 'na' + EXT

def from_status_to_icon_name(status,day = True):
	status = status.lower()
	if day == True:
		if status.find('rain') != -1:
			return 'weather-showers' 
		elif status.find('snow') != -1 or status.find('icy') != -1:
			return 'weather-snow'
		elif status.find('storm') != -1 or status.find('tstorm') != -1 or status.find('thunderstorm') != -1:
			return 'weather-storm'
		elif status.find('fog') != -1 or status.find('haze') != -1 or status.find('dust') != -1 or status.find('smoke') != -1:
			return 'weather-fog'
		elif (status.find('mostly') != -1 or status.find('partly') != -1) and status.find('cloudy') != -1: 
			return 'weather-clouds' 
		elif status.find('cloudy') != -1:
			return 'weather-overcast' 
		elif status.find('mostly') != -1 and status.find('sunny') != -1: 
			return 'weather-few-clouds' 
		elif status.find('sunny') != -1:
			return 'weather-clear' 
		elif status.find('flurries') != -1 or status.find('mist') != -1 or status.find('sleet') != -1:
			return 'weather-showers-scattered' 
	else:
		if status.find('rain') != -1:
			return 'weather-showers' 
		elif status.find('snow') != -1 or status.find('icy') != -1:
			return 'weather-snow'
		elif status.find('storm') != -1 or status.find('tstorm') != -1 or status.find('thunderstorm') != -1:
			return 'weather-storm'
		elif status.find('fog') != -1 or status.find('haze') != -1 or status.find('dust') != -1 or status.find('smoke') != -1:
			return 'weather-fog'
		elif (status.find('mostly') != -1 or status.find('partly') != -1) and status.find('cloudy') != -1: 
			return 'weather-clouds-night' 
		elif status.find('cloudy') != -1:
			return 'weather-overcast' 
		elif status.find('mostly') != -1 and status.find('sunny') != -1: 
			return 'weather-few-clouds-night' 
		elif status.find('sunny') != -1:
			return 'weather-clear-night' 
		elif status.find('flurries') != -1 or status.find('mist') != -1 or status.find('sleet') != -1:
			return 'weather-showers-scattered' 
	return 'weather-severe-alert' 
	
if __name__ == "__main__":
	print get_weather_from_google(hl = 'en')
	exit(0)
